#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
// deno-lint-ignore-file no-explicit-any
const dntShim = __importStar(require("./_dnt.shims.js"));
const cac_1 = require("cac");
const index_js_1 = require("./index.js");
const unityChangeset_js_1 = require("./unityChangeset.js");
const groupBy = (arr, key) => arr.reduce((groups, item) => {
    (groups[key(item)] ||= []).push(item);
    return groups;
}, {});
const cli = (0, cac_1.cac)("unity-changeset");
cli.command("<version>", "Get a changeset for specific version")
    .action((version) => (async () => {
    try {
        const changeset = await (0, index_js_1.getUnityChangeset)(version);
        console.log(changeset.changeset);
    }
    catch {
        console.error("The given version was not found.");
        dntShim.Deno.exit(1);
    }
})());
cli.command("list", "List changesets")
    .option("--min <version>", "Minimum version (included)")
    .option("--max <version>", "Maximum version (included)")
    .option("--grep <version>", "Grep version")
    .option("--json", "Output in json format")
    .option("--pretty-json", "Output in pretty json format")
    .option("--all", "List all changesets (alpha/beta included)")
    .option("--beta", "List alpha/beta changesets")
    .option("--versions", "Output only the available Unity versions")
    .option("--minor-versions", "Output only the available Unity minor versions")
    .option("--latest-patch", "Output only the latest Unity patch versions")
    .option("--oldest-patch", "Output only the oldest Unity patch versions")
    .option("--latest-lifecycle", "Output only the latest lifecycle Unity patch versions")
    .action((options) => (async () => {
    let results = options.all
        ? (await (0, index_js_1.scrapeArchivedChangesets)()).concat(await (0, index_js_1.scrapeBetaChangesets)())
        : options.beta
            ? await (0, index_js_1.scrapeBetaChangesets)()
            : await (0, index_js_1.scrapeArchivedChangesets)();
    // Filter by min/max.
    const min = options.min
        ? unityChangeset_js_1.UnityChangeset.toNumber(options.min, false)
        : Number.MIN_VALUE;
    const max = options.max
        ? unityChangeset_js_1.UnityChangeset.toNumber(options.max, true)
        : Number.MAX_VALUE;
    results = results
        .filter((r) => options.grep ? r.version.includes(options.grep) : true)
        .filter((r) => min <= r.versionNumber && r.versionNumber <= max);
    // Group by minor version
    if (options.minorVersions) {
        results.forEach((r) => r.version = r.minor);
        results = Object.values(groupBy(results, (r) => r.version)).map((g) => g[0]);
    } // Group by minor version and get latest lifecycle patch
    else if (options.latestLifecycle) {
        results = Object.values(groupBy(results, (r) => r.minor))
            .map((g) => g.filter((v) => v.lifecycle == g[0].lifecycle)[0]);
    } // Group by minor version and get latest patch
    else if (options.latestPatch) {
        results = Object.values(groupBy(results, (r) => r.minor)).map((g) => g[0]);
    } // Group by minor version and get oldest patch
    else if (options.oldestPatch) {
        results = Object.values(groupBy(results, (r) => r.minor)).map((g) => g[g.length - 1]);
    }
    // If the result is empty, do not output anything
    if (results.length == 0) {
        return;
    }
    const res = options.versions || options.minorVersions
        ? results.map((r) => r.version)
        : results;
    // Output in json format or plain
    if (options.prettyJson) {
        console.log(JSON.stringify(res, null, "  "));
    }
    else if (options.json) {
        console.log(JSON.stringify(res));
    }
    else {
        console.log(res.map((r) => r.toString()).join("\n"));
    }
})());
cli
    .usage("unity-changeset <command> [options]")
    .example("unity-changeset 2020.1.15f1")
    .example("unity-changeset 2021.1.0a7")
    .example("unity-changeset list")
    .example("unity-changeset list --beta")
    .example("unity-changeset list --versions")
    .example("unity-changeset list --versions --all")
    .example("unity-changeset list --versions --all --latest-patch")
    .help();
if (0 == dntShim.Deno.args.length) {
    cli.outputHelp();
    dntShim.Deno.exit(1);
}
cli.parse();
