"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.scrapeBetaChangesets = exports.scrapeArchivedChangesets = exports.getUnityChangeset = void 0;
const dntShim = __importStar(require("./_dnt.shims.js"));
const unityChangeset_js_1 = require("./unityChangeset.js");
const REGEXP_HUB_LINKS = /unityhub:\/\/\d{4}\.\d+\.\d+(a|b|f)\d+\/\w{12}/g;
const UNITY_ARCHIVE_URL = "https://unity3d.com/get-unity/download/archive";
const UNITY_ALPHA_URL = "https://unity3d.com/unity/alpha/";
const UNITY_BETA_URL = "https://unity3d.com/unity/beta/";
const UNITY_RSS_URL = "https://unity.com/releases/editor/releases.xml";
const UNITY_BETA_RSS_URL = "https://unity3d.com/unity/beta/latest.xml";
/*
 * Get an Unity changeset from specific Unity version.
 * @param version The Unity version.
 * @returns An Unity changeset.
 */
async function getUnityChangeset(version) {
    const match = version.match(/^\d{4}\.\d+\.\d+(a|b|f)\d+$/);
    const lifecycle = match?.[1];
    switch (lifecycle) {
        case "f":
            return (await scrapeArchivedChangesets())
                .filter((c) => c.version === version)[0];
        case "a":
            return (await getUnityChangesetsFromUrl(UNITY_ALPHA_URL + version))
                .filter((c) => c.version === version)[0];
        case "b":
            return (await getUnityChangesetsFromUrl(UNITY_BETA_URL + version))
                .filter((c) => c.version === version)[0];
        default:
            throw Error(`The given life-cycle '${lifecycle}' (in ${version}) was not supported`);
    }
}
exports.getUnityChangeset = getUnityChangeset;
/*
 * Scrape the archived Unity changesets from Unity archives.
 * @returns The Unity changesets.
 */
async function scrapeArchivedChangesets() {
    const changesets = (await getUnityChangesetsFromUrl(UNITY_ARCHIVE_URL))
        .concat(await getUnityChangesetsFromUrl(UNITY_RSS_URL));
    const unique = new Set();
    return changesets.filter((c) => {
        const duplicated = unique.has(c.versionNumber);
        unique.add(c.versionNumber);
        return !duplicated;
    })
        .sort((a, b) => b.versionNumber - a.versionNumber);
}
exports.scrapeArchivedChangesets = scrapeArchivedChangesets;
/*
 * Scrape the alpha/beta Unity changesets from Unity RSS feed.
 * @returns The Unity changesets (alpha/beta).
 */
function scrapeBetaChangesets() {
    return getUnityChangesetsFromUrl(UNITY_BETA_RSS_URL);
}
exports.scrapeBetaChangesets = scrapeBetaChangesets;
async function getUnityChangesetsFromUrl(url) {
    const response = await dntShim.fetch(url);
    const raw = await response.text();
    const match = raw.match(REGEXP_HUB_LINKS);
    if (!match) {
        throw new Error(`No changesets found at '${url}'`);
    }
    return match.map((m) => unityChangeset_js_1.UnityChangeset.createFromHref(m));
}
